//
//  GameView.swift
//  ArkitDoushiQi
//
//  Created by Enzo JOLYS on 27/05/2024.
//

import SwiftUI
import SpriteKit
import DouShouQiModel

struct GameView: View {
    
    var gameScene:GameScene = GameScene(size: CGSize(width: 940, height: 740))

    @State var msg:Text = Text("Msg")
    
    var body: some View {

        VStack(){
            Text($msg)
            SpriteView(scene: game).task {
 
            gameScene.game.addGameStartedListener { board in startGame()}
            gameScene.game.addGameOverListener { board, result, player in gameOver() }
            gameScene.game.addGameChangedListener { game in gameChange() }
            gameScene.game.addBoardChangedListener { board in boardChange() }

            gameScene.game.addMoveChosenCallbacksListener { board, move, player in moveChose(board: board, move: move, player: player) }
            gameScene.game.addInvalidMoveCallbacksListener { board,move,player,bool in invalidMove(board: board, move: move, player:player, bool:bool)}
            
            gameScene.game.addPlayerNotifiedListener { board, player in
                print("Player notif : \(player.id) à toi de jouer ")
            }
            
            try! await gameScene.game.start() 
            }
        }
    }
    
    // ------ Listener -------- //
    
    mutating func startGame() { print("Start game !") 
        msg = "Start !!"
    }
    func gameOver() { print("Game over !") }
    func gameChange() { print("Game change !") }
    func boardChange() {
        print("Board change !")
        gameScene.displayBoard(board: gameScene.game.board)
    }
    
    func moveChose(board:Board,move:Move,player:Player) {
        print("Move choisi !")
        print("joué par \(player.id)")
        print("Move : \(move.description)")
        print("------------")
    }
    
    func invalidMove(board:Board,move:Move,player:Player,bool:Bool) {
        print("Move invalid ! (\(bool))") // True == Valid / False == Invalid
        print("Player : \(player.id)")
        print("Move : \(move.description)")
        print("------------")
    }
    
    
    // ------------------------- //
    
}

struct GameView_Previews: PreviewProvider {
    static var previews: some View {
        GameView()
    }
}
