//
//  SpriteMoople.swift
//  ArkitDoushiQi
//
//  Created by Enzo JOLYS on 27/05/2024.
//

import Foundation
import SpriteKit
import DouShouQiModel

class SpriteMoople : SKNode {
    
    static let offset = CGPoint(x: -400, y: -300 )
    static let direction = CGVector(dx: 100, dy: 100)
    
    let image:SKSpriteNode
    let ellipse:SKShapeNode = SKShapeNode(circleOfRadius: 40)
    
    weak var refGameScene:GameScene?

    var cellPosition:CGPoint = CGPoint(x: 0, y: 0) {
        didSet {
            self.position.x = SpriteMoople.offset.x + SpriteMoople.direction.dx * cellPosition.x
            self.position.y = SpriteMoople.offset.y + SpriteMoople.direction.dy * cellPosition.y
        }
    }
    
    
    init(nameImage:String,couleur:UIColor){
        ellipse.fillColor = couleur
        image = SKSpriteNode(imageNamed: nameImage)
        
        super.init()
        self.addChild(ellipse)
        self.addChild(image)
    }
    
    
    override var isUserInteractionEnabled: Bool {
            get { return true }
            set {}
        }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        var movePoss: [ move ] =  refGameScene!.game.rules.getMoves(board: refGameScene!.game.board,owner:refGameScene!.game.rules.getNextPlayer())
        for pos in movePoss {
            var rect = SKShapeNode(rectOf: CGSize(width: 90, height: 90))
            rect.position = returnPositionByCellPos(cellX:move.destinationRow,cellY:move.destinationColm)
            refGameScene!.deplacementPossible.add(rect)
            refGameScene!.addChild(rect)
        }
    }
    
    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        self.position = touches.first?.location(in: parent!) ?? CGPoint(x: 0, y: 0)
    }
    
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        let localisation = touches.first?.location(in: parent!)

        // Clean rect 
        for rect in refGameScene!.deplacementPossible { 
            rect.removeChild()
        }
        
        if let localisation = localisation {
            
            if (localisation.x > 400 || localisation.x < -400 || localisation.y < -300 || localisation.y > 300){
                print("Hors limite !")
                return
            }
            let posX = Int(round((localisation.x - (-400)) / 100))
            let posY = Int(round((localisation.y - (-300)) / 100))
            
            // Récupéré le joueur qui doit jouer  
            let player = refGameScene!.game.rules.getNextPlayer()
            if ( player == refGameScene!.game.board.grid[self.cellPosition.x][self.cellPosition.y].piece!.owner ){
                if player is HumanPlayer { // Player humain
                    try! (refGameScene!.game.players[player] as! HumanPlayer).chooseMove(x)
                }
                else { // IA 
                    try! await refGameScene!.game.players[player].chooseMove(in: refGameScene!.game.board, with: refGameScene!.game.rules)
                }
                // Change position de la piece 
                self.cellPosition = CGPoint(x: posX, y: posY)
            }
            else {
                // Ce n'est pas la bonne personne, reset position
                self.position = returnPositionByCellPos(cellX:self.cellPosition.x,cellY:self.cellPosition.y)
            }            
            return
        }
    }
    
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func returnPositionByCellPos(cellX:Int,cellY:Int) -> CGPoint {
        return CGPoint(x: SpriteMoople.offset.x + SpriteMoople.direction.dx * cellX,y: self.position.y = SpriteMoople.offset.y + SpriteMoople.direction.dy * cellY)
    }
}
